using UnityEngine;
using UnityEngine.UI;
using System.Collections.Generic;
using System;

namespace FrogFrog
{
    public class MLSLightGroup : MVRScript
    {
        private JSONStorableStringChooser _alignType;
        private UIDynamicPopup _chooserAlignType;
        private JSONStorableFloat _lightRow;
        private UIDynamicSlider _sliderLightRow;
        private JSONStorableFloat _lightCol;
        private UIDynamicSlider _sliderLightCol;
        private JSONStorableFloat _spacingH1;
        private UIDynamicSlider _sliderSpacingH1;
        private JSONStorableFloat _spacingV1;
        private UIDynamicSlider _sliderSpacingV1;
        private JSONStorableBool _useAltH;
        private UIDynamicToggle _tonggleUseAltH;
        private JSONStorableBool _useAltV;
        private UIDynamicToggle _tonggleUseAltV;
        private JSONStorableFloat _spacingH2;
        private UIDynamicSlider _sliderSpacingH2;
        private JSONStorableFloat _spacingV2;
        private UIDynamicSlider _sliderSpacingV2;
        private JSONStorableFloat _radius;
        private UIDynamicSlider _sliderRadius;
        private JSONStorableFloat _spacingR;
        private UIDynamicSlider _sliderSpacingR;
        private JSONStorableFloat _radianRange;
        private UIDynamicSlider _sliderRadianRange;
        private UIDynamicButton _buttonAddAtom;
        private UIDynamicButton _buttonDelAtom;
        private UIDynamicButton _buttonInitializeHPVL;
        private UIDynamicButton _buttonShowLightAtoms;
        private JSONStorableString _atomList;
        private UIDynamicTextField _textAtomList;
        private List<Atom> _lightList = new List<Atom>();
        private bool _seedIsJustPrepared = false;
        private string _seedName = null;
        private JSONStorableAction buildLishts;
        private JSONStorableAction clearLishts;
        private bool lightsIsBuilded = false;
        private JSONStorableString jsHPVLConf = new JSONStorableString("HPVL Configuration", "");
        private JSONStorableAction jsHPVLConfSave;
        private JSONStorableAction jsHPVLConfRestore;
        private UIDynamicButton _buttonHPVLConfSave;
        private UIDynamicButton _buttonHPVLConfRestore;
        private bool HPVLIsReady = false;
        public override void Init()
        {
            _seedName = $"{containingAtom.uid}_SEED";
            BuildSeed();

            RegisterString(jsHPVLConf);

            List<string> alignType = new List<string> { "Flat", "Round" };
            _alignType = new JSONStorableStringChooser("Align Type", alignType, "Flat", "Align Type") { setCallbackFunction = (val) => { UILogic(val); } };
            RegisterStringChooser(_alignType);
            _chooserAlignType = CreateScrollablePopup(_alignType, false);

            _lightCol = new JSONStorableFloat("Column Count", 5.0f, 1.0f, 50.0f, true);
            _lightCol.setCallbackFunction = (val) => { _lightCol.val = Mathf.Round(val); };
            RegisterFloat(_lightCol);
            _sliderLightCol = CreateSlider(_lightCol);

            _lightRow = new JSONStorableFloat("Row Count", 1.0f, 1.0f, 50.0f, true);
            _lightRow.setCallbackFunction = (val) => { _lightRow.val = Mathf.Round(val); };
            RegisterFloat(_lightRow);
            _sliderLightRow = CreateSlider(_lightRow);

            _spacingH1 = new JSONStorableFloat("Horizontal Spacing", 1.0f, 0.0f, 20.0f, true);
            RegisterFloat(_spacingH1);
            _sliderSpacingH1 = CreateSlider(_spacingH1);

            _spacingV1 = new JSONStorableFloat("Vertical Spacing", 1.0f, 0.0f, 20.0f, true);
            RegisterFloat(_spacingV1);
            _sliderSpacingV1 = CreateSlider(_spacingV1);

            _useAltH = new JSONStorableBool("Use Alt Horizontal Spacing", false, (val) => { _spacingH2.SetInteractble(val); });
            RegisterBool(_useAltH);
            _tonggleUseAltH = CreateToggle(_useAltH);

            _spacingH2 = new JSONStorableFloat("Horizontal Spacing Alt", 1.0f, 0.0f, 20.0f, true, _useAltH.val);
            RegisterFloat(_spacingH2);
            _sliderSpacingH2 = CreateSlider(_spacingH2);

            _useAltV = new JSONStorableBool("Use Alt Vertical Spacing", false, (val) => { _spacingV2.SetInteractble(val); });
            RegisterBool(_useAltV);
            _tonggleUseAltV = CreateToggle(_useAltV);

            _spacingV2 = new JSONStorableFloat("Vertical Spacing Alt", 1.0f, 0.0f, 20.0f, true, _useAltV.val);
            RegisterFloat(_spacingV2);
            _sliderSpacingV2 = CreateSlider(_spacingV2);

            _radius = new JSONStorableFloat("Radius", 1.0f, 0.0f, 20.0f, true);
            RegisterFloat(_radius);
            _sliderRadius = CreateSlider(_radius);

            _spacingR = new JSONStorableFloat("Radial Spacing", 1.0f, 0.0f, 20.0f, true);
            RegisterFloat(_spacingR);
            _sliderSpacingR = CreateSlider(_spacingR);

            _radianRange = new JSONStorableFloat("Radian Range", 360.0f, 0.0f, 360.0f, true);
            _radianRange.setCallbackFunction = (val) => { _radianRange.val = Mathf.Round(val); };
            RegisterFloat(_radianRange);
            _sliderRadianRange = CreateSlider(_radianRange);

            UILogic(_alignType.val);

            _buttonAddAtom = CreateButton("Build Light Group", true);
            _buttonAddAtom.button.onClick.AddListener(BuildLightGroup);
            _buttonAddAtom.button.interactable = true;

            _buttonDelAtom = CreateButton("Clear Light Group", true);
            _buttonDelAtom.button.onClick.AddListener(ClearLightGroup);
            _buttonDelAtom.button.interactable = false;

            _buttonShowLightAtoms = CreateButton("Show Light Atoms", true);
            _buttonShowLightAtoms.button.onClick.AddListener(ShowLightAtoms);

            _atomList = new JSONStorableString("AtomList", "");
            _textAtomList = CreateTextField(_atomList, true);
            _textAtomList.height = 450F;

            _buttonInitializeHPVL = CreateButton("Initialize HPVL (After Reloaded)", true);
            _buttonInitializeHPVL.button.onClick.AddListener(InitHPVL);
            _buttonInitializeHPVL.buttonColor = Color.green;

            _buttonHPVLConfSave = CreateButton("Manually Save HPVL Settings", true);
            _buttonHPVLConfSave.button.onClick.AddListener(GetHPVLConf);
            _buttonHPVLConfSave.buttonColor = Color.green;

            buildLishts = new JSONStorableAction("Build Lights", BuildLightGroup);
            RegisterAction(buildLishts);
            clearLishts = new JSONStorableAction("Clear Lights", ClearLightGroup);
            RegisterAction(clearLishts);
            jsHPVLConfSave = new JSONStorableAction("Save HPVL Settings", GetHPVLConf);
            RegisterAction(jsHPVLConfSave);
            jsHPVLConfRestore = new JSONStorableAction("Restore HPVL Settings", SetHPVLConf);
            RegisterAction(jsHPVLConfRestore);

        }

        private void BuildLightGroup()
        {
            _buttonAddAtom.button.interactable = false;

            if (!lightsIsBuilded)
            {
                Atom theSeedAtom = SuperController.singleton.GetAtomByUid(_seedName);
                JSONStorable tSAPhysics = theSeedAtom.GetStorableByID("PhysicsMaterialControl");
                if (tSAPhysics.GetFloatJSONParam("staticFriction").val < 0.2023F || tSAPhysics.GetFloatJSONParam("staticFriction").val > 0.2025F)
                {
                    theSeedAtom.transform.localPosition = new Vector3(0f, 0f, 0f);
                    theSeedAtom.transform.localEulerAngles = new Vector3(0f, 0f, 0f);
                    FreeControllerV3 atomController = theSeedAtom.GetStorableByID("control") as FreeControllerV3;
                    atomController.canGrabPosition = false;
                    atomController.canGrabRotation = false;
                    theSeedAtom.hidden = true;
                    tSAPhysics.GetFloatJSONParam("staticFriction").val = 0.2024F;
                }

                FreeControllerV3 CAController = containingAtom.GetStorableByID("control") as FreeControllerV3;
                int col = (int)(_lightCol.val + 0.1);
                int row = (int)(_lightRow.val + 0.1);
                int range = (int)(_radianRange.val + 0.1);

                SuperController.singleton.PauseSyncAtomLists();
                if (_alignType.val == "Flat")
                {
                    float spacingV = 0.0f;
                    for (int i = 0; i < row; i++)
                    {
                        float spacingH = 0.0f;
                        for (int j = 0; j < col; j++)
                        {
                            Atom tempAtom = SuperController.singleton.GetAtomByUid(_seedName);
                            tempAtom = Instantiate(tempAtom, CAController.transform, false);
                            tempAtom.SetParentAtom(containingAtom);

                            FreeControllerV3 atomController = tempAtom.GetStorableByID("control") as FreeControllerV3;
                            atomController.transform.localPosition = new Vector3(-spacingH, 0f, -spacingV);
                            atomController.transform.localEulerAngles = new Vector3(90.0f, 0.0f, 0.0f);
                            atomController.canGrabPosition = false;
                            atomController.canGrabRotation = false;

                            string SN = _lightList.Count.ToString();
                            while (SN.Length < 4) { SN = "0" + SN; }

                            tempAtom.uid = $"{containingAtom.uid}_{SN}";
                            tempAtom.name = $"{containingAtom.uid}_{SN}_0";

                            _lightList.Add(tempAtom);
                            if (_useAltH.val && j % 2 > 0) spacingH += _spacingH2.val;
                            else spacingH += _spacingH1.val;
                        }
                        if (_useAltV.val && i % 2 > 0) spacingV += _spacingV2.val;
                        else spacingV += _spacingV1.val;
                    }
                }
                else
                {
                    float radius = _radius.val;
                    float angleStep = 0.0F;
                    if (range == 360) angleStep = 360.0f / (float)col;
                    else angleStep = (float)range / (float)(col > 1 ? col - 1 : col);
                    float angleStepMath = angleStep * Mathf.PI / 180.0f;

                    for (int i = 0; i < row; i++)
                    {
                        float angle = 0.0f;
                        float angleMath = Mathf.PI;
                        for (int j = 0; j < col; j++)
                        {
                            Atom tempAtom = SuperController.singleton.GetAtomByUid(_seedName);
                            tempAtom = Instantiate(tempAtom, CAController.transform, false);
                            tempAtom.SetParentAtom(containingAtom);

                            FreeControllerV3 atomController = tempAtom.GetStorableByID("control") as FreeControllerV3;
                            atomController.transform.localPosition = new Vector3(radius * Mathf.Sin(angleMath), 0f, radius * Mathf.Cos(angleMath));
                            atomController.transform.localEulerAngles = new Vector3(90.0f, angle, 0.0f);
                            atomController.canGrabPosition = false;
                            atomController.canGrabRotation = false;

                            string SN = _lightList.Count.ToString();
                            while (SN.Length < 4) { SN = "0" + SN; }

                            tempAtom.uid = $"{containingAtom.uid}_{SN}";
                            tempAtom.name = $"{containingAtom.uid}_{SN}_{angle}";

                            _lightList.Add(tempAtom);
                            angle += angleStep;
                            angleMath += angleStepMath;
                        }
                        radius += _spacingR.val;
                    }
                }
                SuperController.singleton.ResumeSyncAtomLists();
            }

            _buttonDelAtom.button.interactable = true;
            lightsIsBuilded = true;
        }

        private void ClearLightGroup()
        {
            _buttonDelAtom.button.interactable = false;
            if (lightsIsBuilded)
            {
                SuperController.singleton.PauseSyncAtomLists();
                DoClear();
                SuperController.singleton.ResumeSyncAtomLists();
            }
            _buttonAddAtom.button.interactable = true;
            lightsIsBuilded = false;
        }

        private void DoClear()
        {
            List<Atom> lightAtomsList = containingAtom.GetChildren().ToList();
            foreach (var light in lightAtomsList) { SuperController.singleton.RemoveAtom(light); }
            _lightList.Clear();
        }

        private void ShowLightAtoms()
        {
            _atomList.val = "<SIZE=33><B>The Light Atoms: </B></SIZE>";
            string wrapStr = "<size=10>\n\n</size>";
            _atomList.val += wrapStr;
            List<Atom> theList = containingAtom.GetChildren().ToList();
            foreach (var light in theList)
            {
                _atomList.val += light.uid;
                _atomList.val += wrapStr;
            }
        }

        private void BuildSeed()
        {
            if (!(bool)SuperController.singleton.GetAtomByUid(_seedName))
            {
                SuperController.singleton.StartCoroutine(SuperController.singleton.AddAtomByType("Empty", _seedName, false, false, false));
                //_seedIsJustPrepared = true;
            }
        }

        private void UILogic(string ATP)
        {
            if (ATP == "Flat")
            {
                UIDynamicHideAndShow(_sliderSpacingH1, true);
                UIDynamicHideAndShow(_sliderSpacingV1, true);
                UIDynamicHideAndShow(_tonggleUseAltH, true);
                UIDynamicHideAndShow(_sliderSpacingH2, true);
                UIDynamicHideAndShow(_tonggleUseAltV, true);
                UIDynamicHideAndShow(_sliderSpacingV2, true);
                UIDynamicHideAndShow(_sliderRadius, false);
                UIDynamicHideAndShow(_sliderSpacingR, false);
                UIDynamicHideAndShow(_sliderRadianRange, false);
            }
            else
            {
                UIDynamicHideAndShow(_sliderSpacingH1, false);
                UIDynamicHideAndShow(_sliderSpacingV1, false);
                UIDynamicHideAndShow(_tonggleUseAltH, false);
                UIDynamicHideAndShow(_sliderSpacingH2, false);
                UIDynamicHideAndShow(_tonggleUseAltV, false);
                UIDynamicHideAndShow(_sliderSpacingV2, false);
                UIDynamicHideAndShow(_sliderRadius, true);
                UIDynamicHideAndShow(_sliderSpacingR, true);
                UIDynamicHideAndShow(_sliderRadianRange, true);
            }
        }
        private void UIDynamicHideAndShow(UIDynamic UIObj, bool show)
        {
            if (show)
            {
                UIObj.GetComponent<LayoutElement>().transform.localScale = new Vector3(1, 1, 1);
                UIObj.GetComponent<LayoutElement>().ignoreLayout = false;
            }
            else
            {
                UIObj.GetComponent<LayoutElement>().transform.localScale = new Vector3(0, 0, 0);
                UIObj.GetComponent<LayoutElement>().ignoreLayout = true;
            }
        }
        private void InitHPVL()
        {
            List<string> storableIDs = containingAtom.GetStorableIDs();
            foreach (var thisID in storableIDs)
            {
                if (thisID.Contains("HPVL"))
                {
                    JSONStorable theHPVL = containingAtom.GetStorableByID(thisID);
                    theHPVL.CallAction("Initialize HPVL");
                    SetHPVLConf();
                }
            }
        }
        private void GetHPVLConf()
        {
            if (!HPVLIsReady) return;
            List<string> storableIDs = containingAtom.GetStorableIDs();
            foreach (var thisID in storableIDs)
            {
                if (thisID.Contains("HPVL"))
                {
                    JSONStorable theHPVL = containingAtom.GetStorableByID(thisID);
                    if (theHPVL.GetAllParamAndActionNames().Count < 50) return;
                    List<string> boolParamList = theHPVL.GetBoolParamNames();
                    List<string> floatParamList = theHPVL.GetFloatParamNames();
                    List<string> colorParamList = theHPVL.GetColorParamNames();
                    List<string> chooserParamList = theHPVL.GetStringChooserParamNames();
                    jsHPVLConf.val = "";

                    foreach (var boolName in boolParamList)
                    {
                        jsHPVLConf.val += $"BOOL:{boolName}:{theHPVL.GetBoolJSONParam(boolName).val},";
                    }
                    foreach (var floatName in floatParamList)
                    {
                        jsHPVLConf.val += $"FLOAT:{floatName}:{theHPVL.GetFloatJSONParam(floatName).val},";
                    }
                    foreach (var colorName in colorParamList)
                    {
                        HSVColor HSV = theHPVL.GetColorJSONParam(colorName).val;
                        jsHPVLConf.val += $"COLOR:{colorName}:{HSV.H}:{HSV.S}:{HSV.V},";
                    }
                    foreach (var chooserName in chooserParamList)
                    {
                        jsHPVLConf.val += $"CHOOSER:{chooserName}:{theHPVL.GetStringChooserJSONParam(chooserName).val},";
                    }
                    //SuperController.LogMessage(jsHPVLConf.val);
                    return;
                }
            }
        }
        private void SetHPVLConf()
        {
            List<string> storableIDs = containingAtom.GetStorableIDs();
            foreach (var thisID in storableIDs)
            {
                if (thisID.Contains("HPVL"))
                {
                    JSONStorable theHPVL = containingAtom.GetStorableByID(thisID);
                    if (theHPVL.GetAllParamAndActionNames().Count < 50) return;
                    if (jsHPVLConf.val != "")
                    {
                        char[] CSP1 = { ',' };
                        char[] CSP2 = { ':' };
                        string[] paramConfArr = jsHPVLConf.val.Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
                        foreach (var paramConf in paramConfArr)
                        {
                            string[] confItem = paramConf.Split(CSP2, StringSplitOptions.None);
                            if (confItem[0] == "BOOL") theHPVL.GetBoolJSONParam(confItem[1]).val = Convert.ToBoolean(confItem[2]);
                            else if (confItem[0] == "FLOAT") theHPVL.GetFloatJSONParam(confItem[1]).val = Convert.ToSingle(confItem[2]);
                            else if (confItem[0] == "CHOOSER") theHPVL.GetStringChooserJSONParam(confItem[1]).val = confItem[2];
                            else if (confItem[0] == "COLOR")
                            {
                                float H = Convert.ToSingle(confItem[2]);
                                float S = Convert.ToSingle(confItem[3]);
                                float V = Convert.ToSingle(confItem[4]);
                                theHPVL.GetColorJSONParam(confItem[1]).SetVal(H, S, V);
                            }
                        }
                    }
                    HPVLIsReady = true;
                    return;
                }
            }
        }
    }
}