using System;
using System.Collections.Generic;

namespace FrogFrog
{
    public class ConfMain
    {
        public List<ConfAGroups> AGroups = new List<ConfAGroups>();
        public string MLSConfStr = string.Empty;
        public NavInfoStru AddAGroup()
        {
            NavInfoStru thisInfo = new NavInfoStru($"AGroup{AGroups.Count}", AGroups.Count);
            AGroups.Add(new ConfAGroups() { name = thisInfo.label, comment = thisInfo.label });
            return thisInfo;
        }
        public NavInfoStru RemoveAGroup()
        {
            AGroups.RemoveAt(AGroups.Count - 1);
            NavInfoStru thisInfo = new NavInfoStru($"AGroup{AGroups.Count - 1}", AGroups.Count - 1);
            return thisInfo;
        }
        public void ParseStrToConf(string thisStr)
        {
            if (string.IsNullOrEmpty(thisStr.Trim())) return;
            AGroups.Clear();
            string[] SSP = { "<AGroup:>" };
            List<string> confStrList = thisStr.Split(SSP, StringSplitOptions.RemoveEmptyEntries).ToList();

            foreach (var confStr in confStrList)
            {
                AddAGroup();
                AGroups[AGroups.Count - 1].ParseStrToConf(confStr);
            }
        }
        public string ParseConfToStr()
        {
            MLSConfStr = string.Empty;
            foreach (var AGroup in AGroups) { MLSConfStr += $"<AGroup:>{AGroup.ParseConfToStr()}"; }
            return MLSConfStr;
        }
        public void UpdateConf()
        {

        }
    }

    public class ConfAGroups
    {
        public List<ConfActions> Actions = new List<ConfActions>();
        public string name = string.Empty;
        public string comment = string.Empty;
        public NavInfoStru AddAction()
        {
            NavInfoStru thisInfo = new NavInfoStru($"Action{Actions.Count}", Actions.Count);
            Actions.Add(new ConfActions() { name = thisInfo.label, comment = thisInfo.label });
            return thisInfo;
        }
        public NavInfoStru RemoveAction()
        {
            Actions.RemoveAt(Actions.Count - 1);
            NavInfoStru thisInfo = new NavInfoStru($"Action{Actions.Count - 1}", Actions.Count - 1);
            return thisInfo;
        }
        public string ParseConfToStr()
        {
            string resStr = $"comment:{comment}";
            foreach (var Action in Actions) { resStr += $"<Action:>{Action.ParseConfToStr()}"; }
            return resStr;
        }
        public void ParseStrToConf(string thisStr)
        {
            if (string.IsNullOrEmpty(thisStr.Trim())) return;

            string[] SSP = { "<Action:>" };
            List<string> confStrList = thisStr.Split(SSP, StringSplitOptions.RemoveEmptyEntries).ToList();

            char[] CSP1 = { ',' };
            char[] CSP2 = { ':' };
            string[] paramArr = confStrList[0].Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
            confStrList.RemoveAt(0);

            foreach (var paramStr in paramArr)
            {
                string[] paramPair = paramStr.Split(CSP2, StringSplitOptions.None);
                if (paramPair[0] == "comment") comment = paramPair[1];
            }

            foreach (var confStr in confStrList)
            {
                AddAction();
                Actions[Actions.Count - 1].ParseStrToConf(confStr);
            }
        }
    }

    public class ConfActions
    {
        public List<ConfPMotions> GMPs = new List<ConfPMotions>();
        public List<ConfPMotions> LMPs = new List<ConfPMotions>();
        public string name = string.Empty;
        public string comment = string.Empty;
        public string targetGps = string.Empty;
        public string targetLgts = string.Empty;
        public bool useCP = true;
        public Dictionary<string, int> GPMIndex = new Dictionary<string, int>();
        public Dictionary<string, int> LPMIndex = new Dictionary<string, int>();
        public ConfActions()
        {
            //List<string> GPList = new List<string> { "Angle", "Intensity", "Radius", "FadeEnd", "ColorH", "ColorS", "ColorV" };
            foreach (var pName in GlobalConst.GPList)
            {
                GMPs.Add(new ConfPMotions { name = pName });
                GPMIndex.Add(pName, GMPs.Count - 1);
            }
            //List<string> LPList = new List<string> { "Pitch", "Yaw", "YawAlt", "Switch", "Lightness" };
            foreach (var pName in GlobalConst.LPList)
            {
                LMPs.Add(new ConfPMotions { name = pName });
                LPMIndex.Add(pName, LMPs.Count - 1);
            }
        }
        public string ParseConfToStr()
        {
            string resStr = $"comment:{comment},targetGps:{targetGps},targetLgts:{targetLgts},useCP:{useCP}";
            resStr += "<GPMs:>";
            foreach (var pMotion in GMPs) { if (pMotion.isActive) resStr += $"<PMotion:>{pMotion.ParseConfToStr()}"; }
            resStr += "<LPMs:>";
            foreach (var pMotion in LMPs) { if (pMotion.isActive) resStr += $"<PMotion:>{pMotion.ParseConfToStr()}"; }
            return resStr;
        }
        public void ParseStrToConf(string thisStr)
        {
            if (string.IsNullOrEmpty(thisStr)) return;

            string[] SSP = { "<GPMs:>", "<LPMs:>" };
            string[] confStrArr = thisStr.Split(SSP, StringSplitOptions.None);

            char[] CSP1 = { ',' };
            char[] CSP2 = { ':' };
            string[] paramArr = confStrArr[0].Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
            foreach (var paramStr in paramArr)
            {
                string[] paramPair = paramStr.Split(CSP2, StringSplitOptions.None);
                if (paramPair[0] == "comment") comment = paramPair[1];
                else if (paramPair[0] == "targetGps") targetGps = paramPair[1];
                else if (paramPair[0] == "targetLgts") targetLgts = paramPair[1];
                else if (paramPair[0] == "useCP") useCP = Convert.ToBoolean(paramPair[1]);
            }

            string[] SSP1 = { "<PMotion:>" };
            string[] GPMotionArr = confStrArr[1].Split(SSP1, StringSplitOptions.RemoveEmptyEntries);
            foreach (var confStr in GPMotionArr)
            {
                ConfPMotions tempPMotion = new ConfPMotions();
                tempPMotion.ParseStrToConf(confStr);
                GMPs[GPMIndex[tempPMotion.name]] = tempPMotion;
            }

            string[] LPMotionArr = confStrArr[2].Split(SSP1, StringSplitOptions.RemoveEmptyEntries);
            foreach (var confStr in LPMotionArr)
            {
                ConfPMotions tempPMotion = new ConfPMotions();
                tempPMotion.ParseStrToConf(confStr);
                LMPs[LPMIndex[tempPMotion.name]] = tempPMotion;
            }
        }
    }

    public class ConfPMotions
    {
        public List<ConfMNodes> MNodes = new List<ConfMNodes>();
        public string name = string.Empty;
        public bool isActive = false;
        public bool loopPlay = false;
        public string motionType = "Process"; //"Event","Process","Fellow"
        public string refValue = string.Empty;
        public ConfGPItem waitFor = new ConfGPItem();
        public ConfGPItem warmUp = new ConfGPItem();
        public ConfGPItem coolDown = new ConfGPItem();
        public NavInfoStru AddMNode()
        {
            NavInfoStru thisInfo = new NavInfoStru($"MNode{MNodes.Count}", MNodes.Count);
            MNodes.Add(new ConfMNodes() { name = thisInfo.label });
            return thisInfo;
        }
        public NavInfoStru RemoveMNode()
        {
            MNodes.RemoveAt(MNodes.Count - 1);
            NavInfoStru thisInfo = new NavInfoStru($"MNode{MNodes.Count - 1}", MNodes.Count - 1);
            return thisInfo;
        }
        public string ParseConfToStr()
        {
            string resStr = $"name:{name},isActive:{isActive},loopPlay:{loopPlay},motionType:{motionType},refValue:{refValue}";
            resStr += $"<waitFor:>{waitFor.ParseConfToStr()}";
            resStr += $"<warmUp:>{warmUp.ParseConfToStr()}";
            resStr += $"<coolDown:>{coolDown.ParseConfToStr()}";
            foreach (var MNode in MNodes) { resStr += $"<MNode:>{MNode.ParseConfToStr()}"; }
            return resStr;
        }
        public void ParseStrToConf(string thisStr)
        {
            if (string.IsNullOrEmpty(thisStr.Trim())) return;

            string[] SSP = { "<waitFor:>", "<warmUp:>", "<coolDown:>", "<MNode:>" };
            List<string> confStrList = thisStr.Split(SSP, StringSplitOptions.RemoveEmptyEntries).ToList();

            char[] CSP1 = { ',' };
            char[] CSP2 = { ':' };
            string[] paramArr = confStrList[0].Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
            confStrList.RemoveAt(0);

            foreach (var paramStr in paramArr)
            {
                string[] paramPair = paramStr.Split(CSP2, StringSplitOptions.None);
                if (paramPair[0] == "name") name = paramPair[1];
                else if (paramPair[0] == "isActive") isActive = Convert.ToBoolean(paramPair[1]);
                else if (paramPair[0] == "loopPlay") loopPlay = Convert.ToBoolean(paramPair[1]);
                else if (paramPair[0] == "motionType") motionType = paramPair[1];
                else if (paramPair[0] == "refValue") refValue = paramPair[1];
            }

            waitFor.ParseStrToConf(confStrList[0]);
            confStrList.RemoveAt(0);
            warmUp.ParseStrToConf(confStrList[0]);
            confStrList.RemoveAt(0);
            coolDown.ParseStrToConf(confStrList[0]);
            confStrList.RemoveAt(0);

            foreach (var confStr in confStrList)
            {
                AddMNode();
                MNodes[MNodes.Count - 1].ParseStrToConf(confStr);
            }
        }
    }

    public class ConfMNodes
    {
        public ConfGPItem time = new ConfGPItem();
        public ConfGPItem value = new ConfGPItem();
        public string name = string.Empty;
        public string speedMode = "Normal";
        public bool setTrigger = false;
        public bool isRepeatable = false;
        public string traceAtom = string.Empty;
        public int threshold = 1;
        public bool isInternal = true;
        public string doAction = string.Empty;
        public string ParseConfToStr()
        {
            string resStr = $"setTrigger:{setTrigger},speedMode:{speedMode}";
            resStr += $"<time:>{time.ParseConfToStr()}";
            resStr += $"<value:>{value.ParseConfToStr()}";
            resStr += "<trigger:>";
            if (setTrigger) resStr += $"isRepeatable:{isRepeatable},traceAtom:{traceAtom},threshold:{threshold},isInternal:{isInternal},doAction:{doAction}";
            return resStr;
        }
        public void ParseStrToConf(string thisStr)
        {
            if (string.IsNullOrEmpty(thisStr.Trim())) return;

            string[] SSP = { "<time:>", "<value:>", "<trigger:>" };
            string[] confStrArr = thisStr.Split(SSP, StringSplitOptions.None);

            char[] CSP1 = { ',' };
            char[] CSP2 = { ':' };

            string[] paramArr = confStrArr[0].Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
            foreach (var paramStr in paramArr)
            {
                string[] paramPair = paramStr.Split(CSP2, StringSplitOptions.None);
                if (paramPair[0] == "setTrigger") setTrigger = Convert.ToBoolean(paramPair[1]);
                if (paramPair[0] == "speedMode") speedMode = paramPair[1];
            }

            time.ParseStrToConf(confStrArr[1]);
            value.ParseStrToConf(confStrArr[2]);

            if (setTrigger)
            {
                string[] tParamArr = confStrArr[3].Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
                foreach (var tParamStr in tParamArr)
                {
                    string[] tParamPair = tParamStr.Split(CSP2, StringSplitOptions.None);
                    if (tParamPair[0] == "isRepeatable") isRepeatable = Convert.ToBoolean(tParamPair[1]);
                    else if (tParamPair[0] == "traceAtom") traceAtom = tParamPair[1];
                    else if (tParamPair[0] == "threshold") threshold = Convert.ToInt32(tParamPair[1]);
                    else if (tParamPair[0] == "isInternal") isInternal = Convert.ToBoolean(tParamPair[1]);
                    else if (tParamPair[0] == "doAction") doAction = tParamPair[1];
                }
            }
        }
    }

    public class ConfGPItem
    {
        public string type = "Fixed"; //"Fixed","Random","Incremental"
        public float val1 = 0.0f;
        public float val2 = 0.0f;
        //public float valFinal = 0.0f;
        public string ParseConfToStr()
        {
            return $"type:{type},val1:{val1},val2:{val2}";
        }
        public void ParseStrToConf(string thisStr)
        {
            if (string.IsNullOrEmpty(thisStr.Trim())) return;

            char[] CSP1 = { ',' };
            char[] CSP2 = { ':' };
            string[] paramArr = thisStr.Split(CSP1, StringSplitOptions.RemoveEmptyEntries);
            foreach (var paramStr in paramArr)
            {
                string[] paramPair = paramStr.Split(CSP2, StringSplitOptions.None);
                if (paramPair[0] == "type") type = paramPair[1];
                else if (paramPair[0] == "val1") val1 = Convert.ToSingle(paramPair[1]);
                else if (paramPair[0] == "val2") val2 = Convert.ToSingle(paramPair[1]);
                //else if (paramPair[0] == "valFinal") valFinal = Convert.ToSingle(paramPair[1]);
            }
        }
    }
}