using UnityEngine;
using System;
using System.Collections.Generic;
using System.Linq;

namespace FrogFrog
{
    public class MLSMotion
    {
        public Dictionary<string, MLSGroup> mlsGroupDict = new Dictionary<string, MLSGroup>();
        private Dictionary<string, int> incrCountDict = new Dictionary<string, int>();
        public JSONStorable theMLSCC = null;
        public JSONStorable theVUML = null;
        public JSONStorable theSelectChoice = null;
        private string currAtom = "";
        private int currActIndex = -1;
        private string currProperty = "";
        private string currItem = "";
        public System.Random randomObj = new System.Random();
        public float frameTime = 0.01F;

        public void RefleshMLSAtoms()
        {
            mlsGroupDict.Clear();
            List<Atom> allAtoms = SuperController.singleton.GetAtoms();
            foreach (var thisAtom in allAtoms)
            {
                List<string> storableIDs = thisAtom.GetStorableIDs();
                string[] findPlugin = new string[2];
                foreach (var thisID in storableIDs)
                {
                    if (thisID.Contains("MLSLightGroup")) findPlugin[0] = thisID;
                    else if (thisID.Contains("HPVL")) findPlugin[1] = thisID;
                }
                if (!string.IsNullOrEmpty(findPlugin[0]))
                {
                    MLSGroup mlsGroup = new MLSGroup
                    {
                        MLSLightGroup = thisAtom.GetStorableByID(findPlugin[0]),
                        name = thisAtom.uid,
                    };
                    if (!string.IsNullOrEmpty(findPlugin[1])) mlsGroup.HPVL = thisAtom.GetStorableByID(findPlugin[1]);
                    else { SuperController.LogError($"Can not find the HPVL plugin on atom {thisAtom.uid}"); return; }

                    Dictionary<string, Atom> theChildrenDict = thisAtom.GetChildren().ToDictionary(p => p.uid);
                    theChildrenDict = theChildrenDict.OrderBy(pair => pair.Key).ToDictionary(pair => pair.Key, pair => pair.Value);
                    foreach (var KVP in theChildrenDict)
                    {
                        char[] SPL = { '_' };
                        string[] offSetInfo = KVP.Value.name.Split(SPL, StringSplitOptions.None);
                        MLSLight thisLgt = new MLSLight
                        {
                            lgtControl = KVP.Value.GetStorableByID("control") as FreeControllerV3,
                            lgtPhysics = KVP.Value.GetStorableByID("PhysicsMaterialControl"),
                            lgtSwitch = KVP.Value.GetStorableByID("AtomControl"),

                            name = KVP.Value.uid,
                            YawOffSet = Convert.ToSingle(offSetInfo[2]),
                        };
                        mlsGroup.mlsLightList.Add(thisLgt);
                    }
                    mlsGroupDict.Add(mlsGroup.name, mlsGroup);
                }
            }
            mlsGroupDict = mlsGroupDict.OrderBy(pair => pair.Key).ToDictionary(pair => pair.Key, pair => pair.Value);
        }

        public void LoadAGroup(ConfAGroups AGroup)
        {
            int i = 0;
            foreach (var Action in AGroup.Actions) { currActIndex = i; LoadAction(Action); i++; }
        }

        public void LoadAction(ConfActions Action)
        {
            incrCountDict.Clear();
            char[] SPL = { ';' };
            List<string> tLGpList = new List<string>();
            List<int> tLgtList = new List<int>();
            string tLGpStr = Action.targetGps.Trim().ToUpper();
            string tLgtStr = Action.targetLgts.Trim().ToUpper();
            if (tLGpStr == "ALL" || tLGpStr == "ALLREV")
            {
                foreach (var KVP in mlsGroupDict) { tLGpList.Add(KVP.Key); }
                if (tLGpStr == "ALLREV") tLGpList.Reverse();
            }
            else { tLGpList = Action.targetGps.Split(SPL, StringSplitOptions.RemoveEmptyEntries).ToList(); }

            if (tLgtStr != "ALL" && tLgtStr != "ALLREV")
            {
                string[] tLgtArrStr = Action.targetLgts.Split(SPL, StringSplitOptions.RemoveEmptyEntries);
                foreach (var SN in tLgtArrStr) tLgtList.Add(Convert.ToInt16(SN));
            }

            Dictionary<string, int> GPx = Action.GPMIndex;
            Dictionary<string, int> LPx = Action.LPMIndex;
            foreach (var tLGp in tLGpList)
            {
                currAtom = tLGp;
                if (mlsGroupDict.ContainsKey(tLGp))
                {
                    if (Action.GMPs[GPx["Angle"]].isActive) mlsGroupDict[tLGp].Angle = LoadProperty(Action.GMPs[GPx["Angle"]]);
                    if (Action.GMPs[GPx["Intensity"]].isActive) mlsGroupDict[tLGp].Intensity = LoadProperty(Action.GMPs[GPx["Intensity"]]);
                    if (Action.GMPs[GPx["Radius"]].isActive) mlsGroupDict[tLGp].Radius = LoadProperty(Action.GMPs[GPx["Radius"]]);
                    if (Action.GMPs[GPx["FadeEnd"]].isActive) mlsGroupDict[tLGp].FadeEnd = LoadProperty(Action.GMPs[GPx["FadeEnd"]]);
                    if (Action.GMPs[GPx["ColorH"]].isActive) mlsGroupDict[tLGp].ColorH = LoadProperty(Action.GMPs[GPx["ColorH"]], Action.useCP);
                    if (Action.GMPs[GPx["ColorS"]].isActive) mlsGroupDict[tLGp].ColorS = LoadProperty(Action.GMPs[GPx["ColorS"]], Action.useCP);
                    if (Action.GMPs[GPx["ColorV"]].isActive) mlsGroupDict[tLGp].ColorV = LoadProperty(Action.GMPs[GPx["ColorV"]], Action.useCP);

                    if (tLgtStr == "ALL" || tLgtStr == "ALLREV")
                    {
                        int i = 0;
                        tLgtList.Clear();
                        foreach (var mlsLight in mlsGroupDict[tLGp].mlsLightList) { tLgtList.Add(i); i++; }
                        if (tLgtStr == "ALLREV") tLgtList.Reverse();
                    }

                    foreach (var tLgt in tLgtList)
                    {
                        if (mlsGroupDict[tLGp].mlsLightList.Count > tLgt)
                        {
                            currAtom = mlsGroupDict[tLGp].mlsLightList[tLgt].name;
                            if (Action.LMPs[LPx["Pitch"]].isActive) mlsGroupDict[tLGp].mlsLightList[tLgt].Pitch = LoadProperty(Action.LMPs[LPx["Pitch"]]);
                            if (Action.LMPs[LPx["Yaw"]].isActive) mlsGroupDict[tLGp].mlsLightList[tLgt].Yaw = LoadProperty(Action.LMPs[LPx["Yaw"]]);
                            if (Action.LMPs[LPx["YawAlt"]].isActive) mlsGroupDict[tLGp].mlsLightList[tLgt].YawAlt = LoadProperty(Action.LMPs[LPx["YawAlt"]]);
                            if (Action.LMPs[LPx["Switch"]].isActive) mlsGroupDict[tLGp].mlsLightList[tLgt].Switch = LoadProperty(Action.LMPs[LPx["Switch"]]);
                            if (Action.LMPs[LPx["Lightness"]].isActive) mlsGroupDict[tLGp].mlsLightList[tLgt].Lightness = LoadProperty(Action.LMPs[LPx["Lightness"]]);
                        }
                    }
                }
            }
        }

        public MLSProperty LoadProperty(ConfPMotions pMotion, bool thisUseCP = false)
        {
            currProperty = pMotion.name;
            MLSProperty mlsProperty = new MLSProperty
            {
                MNodes = new List<MLSNode>(),
                name = pMotion.name,
                isActive = pMotion.isActive,
                loopPlay = pMotion.loopPlay,
                motionType = pMotion.motionType,
                refValue = pMotion.refValue,
                useCP = thisUseCP,
                state = 0,
            };
            currItem = "waitFor";
            mlsProperty.waitFor = LoadItem(pMotion.waitFor);
            mlsProperty.waitFor.valFinal = Mathf.Max(mlsProperty.waitFor.valFinal, 0.0F);
            currItem = "warmUp";
            mlsProperty.warmUp = LoadItem(pMotion.warmUp);
            mlsProperty.warmUp.valFinal = Mathf.Max(mlsProperty.warmUp.valFinal, 0.0F);
            currItem = "coolDown";
            mlsProperty.coolDown = LoadItem(pMotion.coolDown);
            mlsProperty.coolDown.valFinal = Mathf.Max(mlsProperty.coolDown.valFinal, 0.0F);
            currItem = "";
            foreach (var mNode in pMotion.MNodes)
            {
                MLSNode newNode = LoadNode(mNode);
                mlsProperty.MNodes.Add(newNode);
            }
            return mlsProperty;
        }

        public MLSNode LoadNode(ConfMNodes mNode)
        {
            MLSNode mlsNode = new MLSNode
            {
                name = mNode.name,
                speedMode = mNode.speedMode,
                isDone = false,
                setTrigger = false,
            };
            currItem = mlsNode.name + "time";
            mlsNode.time = LoadItem(mNode.time);
            mlsNode.time.valFinal = Mathf.Max(mlsNode.time.valFinal, 0.0F);
            currItem = mlsNode.name + "value";
            mlsNode.value = LoadItem(mNode.value);
            currItem = "";

            if (mNode.setTrigger && currAtom == mNode.traceAtom)
            {
                mlsNode.setTrigger = true;
                mlsNode.trigger = new MLSTrigger
                {
                    isRepeatable = mNode.isRepeatable,
                    threshold = mNode.threshold,
                    isInternal = mNode.isInternal,
                    doAction = mNode.doAction,
                    counter = 0,
                    isTriggered = false,
                };
            }
            return mlsNode;
        }

        public MLSItem LoadItem(ConfGPItem gpItem)
        {
            MLSItem mlsItem = new MLSItem
            {
                type = gpItem.type,
                val1 = gpItem.val1,
                val2 = gpItem.val2,
            };
            if (mlsItem.type == "Incremental")
            {
                string countKey = $"{currActIndex}{currProperty}{currItem}";
                if (incrCountDict.ContainsKey(countKey))
                {
                    mlsItem.valFinal = mlsItem.val1 + mlsItem.val2 * incrCountDict[countKey];
                    incrCountDict[countKey]++;
                }
                else
                {
                    mlsItem.valFinal = mlsItem.val1;
                    incrCountDict.Add(countKey, 1);
                }
                //SuperController.LogMessage($"{countKey} = {incrCountDict[countKey]} ::: {mlsItem.valFinal}");
            }
            else if (mlsItem.type == "Fixed") { mlsItem.valFinal = mlsItem.val1; }
            return mlsItem;
        }

        public void PerformMotions()
        {
            frameTime = Time.fixedDeltaTime;
            foreach (var KVP in mlsGroupDict)
            {
                List<string> PNAList = KVP.Value.HPVL.GetAllParamAndActionNames();
                if (PNAList.Count > 50)
                {
                    ParsePMotion(KVP.Value.Angle);
                    if (KVP.Value.Angle.shouldBeSet)
                    {
                        KVP.Value.HPVL.GetFloatJSONParam("Spot Angle").val = Mathf.Clamp(KVP.Value.Angle.currentVal, GlobalConst.min["Angle"], GlobalConst.max["Angle"]);
                        KVP.Value.Angle.shouldBeSet = false;
                    }
                    ParsePMotion(KVP.Value.Intensity);
                    if (KVP.Value.Intensity.shouldBeSet)
                    {
                        KVP.Value.HPVL.GetFloatJSONParam("Intensity Multiplier").val = Mathf.Clamp(KVP.Value.Intensity.currentVal, GlobalConst.min["Intensity"], GlobalConst.max["Intensity"]);
                        KVP.Value.HPVL.GetFloatJSONParam("Light Sphere Intensity Multiper").val = Mathf.Clamp(KVP.Value.Intensity.currentVal * 3.0F, 1.0F, 6.0F);
                        KVP.Value.Intensity.shouldBeSet = false;
                    }
                    ParsePMotion(KVP.Value.Radius);
                    if (KVP.Value.Radius.shouldBeSet)
                    {
                        KVP.Value.HPVL.GetFloatJSONParam("Cone Radius Start").val = Mathf.Clamp(KVP.Value.Radius.currentVal, GlobalConst.min["Radius"], GlobalConst.max["Radius"]);
                        KVP.Value.Radius.shouldBeSet = false;
                    }
                    ParsePMotion(KVP.Value.FadeEnd);
                    if (KVP.Value.FadeEnd.shouldBeSet)
                    {
                        KVP.Value.HPVL.GetFloatJSONParam("Custom Fade End").val = Mathf.Clamp(KVP.Value.FadeEnd.currentVal, GlobalConst.min["FadeEnd"], GlobalConst.max["FadeEnd"]);
                        KVP.Value.FadeEnd.shouldBeSet = false;
                    }
                    ParsePMotion(KVP.Value.ColorH);
                    if (KVP.Value.ColorH.useCP)
                    {
                        ParsePMotion(KVP.Value.ColorS, KVP.Value.ColorH);
                        ParsePMotion(KVP.Value.ColorV, KVP.Value.ColorH);
                    }
                    else
                    {
                        ParsePMotion(KVP.Value.ColorS);
                        ParsePMotion(KVP.Value.ColorV);
                    }
                    if (KVP.Value.ColorH.shouldBeSet || KVP.Value.ColorS.shouldBeSet || KVP.Value.ColorV.shouldBeSet)
                    {
                        float H = Mathf.Clamp(KVP.Value.ColorH.currentVal, GlobalConst.min["ColorH"], GlobalConst.max["ColorH"]);
                        float S = Mathf.Clamp(KVP.Value.ColorS.currentVal, GlobalConst.min["ColorS"], GlobalConst.max["ColorS"]);
                        float V = Mathf.Clamp(KVP.Value.ColorV.currentVal, GlobalConst.min["ColorV"], GlobalConst.max["ColorV"]);
                        KVP.Value.HPVL.GetColorJSONParam("Custom Color").SetVal(H, S, V);
                        KVP.Value.ColorH.shouldBeSet = false;
                        KVP.Value.ColorS.shouldBeSet = false;
                        KVP.Value.ColorV.shouldBeSet = false;
                    }
                }

                foreach (var light in KVP.Value.mlsLightList)
                {
                    ParsePMotion(light.Pitch);
                    ParsePMotion(light.Yaw);
                    ParsePMotion(light.YawAlt);
                    if (light.Pitch.shouldBeSet || light.Yaw.shouldBeSet || light.YawAlt.shouldBeSet)
                    {
                        float P = Mathf.Clamp(light.Pitch.currentVal, GlobalConst.min["Pitch"], GlobalConst.max["Pitch"]);
                        float Y = light.Yaw.currentVal + light.YawOffSet;
                        float A = Mathf.Clamp(light.YawAlt.currentVal, -89.99F, 89.99F);

                        if (P > 90)
                        {
                            Y += 180;
                            P = 180 - P;
                        }
                        if (A != 0)
                        {
                            float PiA = A * Mathf.PI / 180;
                            float PiP = P * Mathf.PI / 180;
                            Y = Mathf.Atan(Mathf.Tan(PiA) / Mathf.Cos(PiP)) * 180 / Mathf.PI + Y;
                            P = Mathf.Asin(Mathf.Cos(PiA) * Mathf.Sin(PiP)) * 180 / Mathf.PI;
                        }
                        Y %= 360;
                        if (Y < 0) { Y += 360; }

                        light.lgtControl.transform.localEulerAngles = new Vector3(P, Y, 0.0F);
                        light.Pitch.shouldBeSet = false;
                        light.Yaw.shouldBeSet = false;
                        light.YawAlt.shouldBeSet = false;
                    }
                    ParsePMotion(light.Switch);
                    if (light.Switch.shouldBeSet)
                    {
                        if (light.Switch.currentVal < 0.5F) light.lgtSwitch.GetBoolJSONParam("on").val = false;
                        else light.lgtSwitch.GetBoolJSONParam("on").val = true;
                        light.Switch.shouldBeSet = false;
                    }
                    ParsePMotion(light.Lightness);
                    if (light.Lightness.shouldBeSet)
                    {
                        light.lgtPhysics.GetFloatJSONParam("dynamicFriction").val = Mathf.Clamp(light.Lightness.currentVal, GlobalConst.min["Lightness"], GlobalConst.max["Lightness"]);
                        light.Lightness.shouldBeSet = false;
                    }
                }
            }
        }
        private void ParsePMotion(MLSProperty property, MLSProperty refProp = null)
        {
            if (property.state > 7) //nothing to do
            {
                //property.shouldBeSet = false;
                return;
            }
            if (property.state == 0) //start waiting
            {
                RefleshValFinal(property.waitFor);
                property.waitingTime = 0.0F;
                property.state = 1;
            }
            if (property.state == 1) //waiting
            {
                property.waitingTime += frameTime;
                if (property.waitingTime >= property.waitFor.valFinal) property.state = 2;
            }
            if (property.state == 2) //start warming up
            {
                int tmpI = 0;
                foreach (var MNode in property.MNodes)
                {
                    if (refProp != null)
                    {
                        RefleshValFinal(MNode.time, refProp.MNodes[tmpI].time);
                        RefleshValFinal(MNode.value, refProp.MNodes[tmpI].value);
                    }
                    else
                    {
                        RefleshValFinal(MNode.time);
                        RefleshValFinal(MNode.value);
                    }
                    MNode.isDone = false;
                    if (tmpI < 1 && MNode.time.valFinal < frameTime)
                    {
                        property.shouldBeSet = true;
                        property.currentVal = MNode.value.valFinal;
                    }
                    tmpI++;
                }
                RefleshValFinal(property.warmUp);
                property.warmingTime = 0.0F;
                property.state = 3;
            }
            if (property.state == 3) //warming up
            {
                property.warmingTime += frameTime;
                if (property.warmingTime >= property.warmUp.valFinal) property.state = 4;
            }
            if (property.state == 4) //start motion
            {
                property.playingTime = 0.0F;
                property.state = 5;
            }
            if (property.state == 5) //motion
            {
                if (property.motionType == "Fellow")
                {
                    if (property.MNodes.Count > 1 && theVUML != null)
                    {
                        float v1 = property.MNodes[0].value.valFinal;
                        float v2 = property.MNodes[1].value.valFinal;
                        float v3 = theVUML.GetFloatJSONParam(property.refValue + "Result").val;
                        property.currentVal = v1 + (v2 - v1) * Mathf.Clamp(v3, 0.0F, 1.0F);
                        property.shouldBeSet = true;
                    }
                }
                else
                {
                    property.playingTime += frameTime;
                    float nextTime = 999999999.0F;
                    float nextVal = 0.0F;
                    string speedMode = "Normal";
                    int MNodesLeft = 0;

                    foreach (var MNode in property.MNodes)
                    {
                        if (property.playingTime >= MNode.time.valFinal)
                        {
                            if (!MNode.isDone)
                            {
                                property.currentVal = MNode.value.valFinal;
                                property.shouldBeSet = true;
                                property.lastTime = MNode.time.valFinal;
                                property.lastVal = MNode.value.valFinal;
                                MNode.isDone = true;
                                if (MNode.setTrigger) ParseTrigger(MNode.trigger);
                            }
                        }
                        else
                        {
                            if (property.motionType == "Process" && nextTime > MNode.time.valFinal)
                            {
                                nextTime = MNode.time.valFinal;
                                nextVal = MNode.value.valFinal;
                                speedMode = MNode.speedMode;
                            }
                            MNodesLeft++;
                        }
                    }

                    if (property.motionType == "Process" && nextTime < 999999999.0F)
                    {
                        float range = nextVal - property.lastVal;
                        float pct = (property.playingTime - property.lastTime) / (nextTime - property.lastTime);
                        if (speedMode == "Accelerate") { pct = Mathf.Clamp(pct * pct, 0, 1); }
                        //else if (speedMode == "Decelerate") { pct = Mathf.Clamp(1 - (1 - pct) * (1 - pct), 0, 1); }
                        else if (speedMode == "Decelerate") { pct = Mathf.Clamp(2 * pct - pct * pct, 0, 1); }

                        property.currentVal = property.lastVal + range * pct;
                        property.shouldBeSet = true;
                    }
                    if (MNodesLeft == 0) property.state = 6;
                }
            }
            if (property.state == 6) //start cooling down
            {
                RefleshValFinal(property.coolDown);
                property.coolingTime = 0.0F;
                property.state = 7;
            }
            if (property.state == 7) //cooling down
            {
                property.coolingTime += frameTime;
                if (property.coolingTime >= property.coolDown.valFinal)
                {
                    if (property.loopPlay) property.state = 2;
                    else property.state = 8;
                }
            }
        }
        private void RefleshValFinal(MLSItem item, MLSItem refItem = null)
        {
            if (item.type == "Random")
            {
                if (refItem != null && refItem.randomSeed >= 0.0f && refItem.randomSeed <= 1.0F) item.randomSeed = refItem.randomSeed;
                else item.randomSeed = (float)randomObj.NextDouble();
                item.valFinal = item.val1 + (item.val2 - item.val1) * item.randomSeed;
            }
        }
        private void ParseTrigger(MLSTrigger trigger)
        {
            if (trigger.isTriggered) return;
            trigger.counter++;
            if (trigger.counter >= trigger.threshold)
            {
                if (trigger.isInternal)
                {
                    if (theMLSCC != null)
                    {
                        theMLSCC.GetStringJSONParam("AGroup Name").val = trigger.doAction;
                        theMLSCC.CallAction("Perform The AGroup");
                    }
                }
                else
                {
                    if (theSelectChoice != null)
                    {
                        theSelectChoice.GetStringJSONParam("SelectChoice").val = trigger.doAction;
                        theSelectChoice.CallAction("Trigger");
                    }
                }
                if (trigger.isRepeatable) trigger.counter = 0;
                else trigger.isTriggered = true;
            }
        }
    }

    public class MLSGroup
    {
        public JSONStorable HPVL;
        public JSONStorable MLSLightGroup;
        public string name = "";
        public List<MLSLight> mlsLightList = new List<MLSLight>();
        public MLSProperty Angle = new MLSProperty();
        public MLSProperty Intensity = new MLSProperty();
        public MLSProperty Radius = new MLSProperty();
        public MLSProperty FadeEnd = new MLSProperty();
        public MLSProperty ColorH = new MLSProperty();
        public MLSProperty ColorS = new MLSProperty();
        public MLSProperty ColorV = new MLSProperty();
        public MLSGroup()
        {
            Angle.currentVal = GlobalConst.def["Angle"];
            Angle.shouldBeSet = true;
            Intensity.currentVal = GlobalConst.def["Intensity"];
            Intensity.shouldBeSet = true;
            Radius.currentVal = GlobalConst.def["Radius"];
            Radius.shouldBeSet = true;
            FadeEnd.currentVal = GlobalConst.def["FadeEnd"];
            FadeEnd.shouldBeSet = true;
            ColorH.currentVal = GlobalConst.def["ColorH"];
            ColorH.shouldBeSet = true;
            ColorS.currentVal = GlobalConst.def["ColorS"];
            ColorS.shouldBeSet = true;
            ColorV.currentVal = GlobalConst.def["ColorV"];
            ColorV.shouldBeSet = true;
        }
    }

    public class MLSLight
    {
        public FreeControllerV3 lgtControl;
        public JSONStorable lgtPhysics;
        public JSONStorable lgtSwitch;
        public string name = "";
        public float YawOffSet = 0.0F;
        public MLSProperty Pitch = new MLSProperty();
        public MLSProperty Yaw = new MLSProperty();
        public MLSProperty YawAlt = new MLSProperty();
        public MLSProperty Switch = new MLSProperty();
        public MLSProperty Lightness = new MLSProperty();
        public MLSLight()
        {
            Pitch.currentVal = GlobalConst.def["Pitch"];
            Pitch.shouldBeSet = true;
            Yaw.currentVal = GlobalConst.def["Yaw"];
            Yaw.shouldBeSet = true;
            YawAlt.currentVal = GlobalConst.def["YawAlt"];
            YawAlt.shouldBeSet = true;
            Switch.currentVal = GlobalConst.def["Switch"];
            Switch.shouldBeSet = true;
            Lightness.currentVal = GlobalConst.def["Lightness"];
            Lightness.shouldBeSet = true;
        }
    }

    public class MLSProperty
    {
        public List<MLSNode> MNodes = new List<MLSNode>();
        public string name = "";
        public bool isActive = false;
        public bool loopPlay = false;
        public string motionType = "";
        public string refValue = "";
        public bool useCP = false;
        public MLSItem waitFor = new MLSItem();
        public MLSItem warmUp = new MLSItem();
        public MLSItem coolDown = new MLSItem();
        public int state = 8;
        public float waitingTime = 0.0F;
        public float warmingTime = 0.0F;
        public float coolingTime = 0.0F;
        public float playingTime = 0.0F;
        public float currentVal = 0.0F;
        public float lastTime = 0.0F;
        public float lastVal = 0.0F;
        public bool shouldBeSet = false;
    }

    public class MLSNode
    {
        public string name = "";
        public string speedMode = "Normal";
        public MLSItem time = new MLSItem();
        public MLSItem value = new MLSItem();
        public bool isDone = false;
        public bool setTrigger = false;
        public MLSTrigger trigger = new MLSTrigger();
    }
    public class MLSItem
    {
        public string type = "Fixed";
        public float val1 = 0.0F;
        public float val2 = 0.0F;
        public float valFinal = 0.0F;
        public float randomSeed = -1.0F;
    }

    public class MLSTrigger
    {
        public bool isRepeatable = false;
        public string traceAtom = "";
        public int threshold = 1;
        public bool isInternal = true;
        public string doAction = "";
        public int counter = 0;
        public bool isTriggered = false;
    }
}