using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Events;
using System.Collections.Generic;

namespace FrogFrog
{
    public static class Utils
    {
        public static JSONStorableBool CreateToggle(MVRScript self, string text, bool startValue, bool rightSide, bool acs)
        {
            JSONStorableBool storable = new JSONStorableBool(text, startValue);
            storable.storeType = JSONStorableParam.StoreType.Full;
            self.CreateToggle(storable, rightSide).label = text;
            self.RegisterBool(storable);
            Toggle toggle = storable.toggle;
            toggle.enabled = acs;
            if (!acs) toggle.transform.Find("Panel").GetComponent<Image>().color = Color.gray;
            return storable;
        }
        public static JSONStorableFloat CreateSliderFloat(MVRScript self, string text, float startValue, float min, float max, bool rightSide, bool acs)
        {
            JSONStorableFloat storable = new JSONStorableFloat(text, startValue, min, max, true, true);
            storable.storeType = JSONStorableParam.StoreType.Full;
            UIDynamicSlider uIDynamicSlider = self.CreateSlider(storable, rightSide);
            uIDynamicSlider.label = text;
            uIDynamicSlider.valueFormat = "F4";
            self.RegisterFloat(storable);
            Slider slider = storable.slider;
            var quickGroup = slider.transform.parent.Find("QuickButtonsGroup").gameObject;
            slider.enabled = acs;
            quickGroup.SetActive(acs);
            if (!acs) slider.transform.parent.Find("Panel").GetComponent<Image>().color = Color.gray;
            return storable;
        }
        public static JSONStorableFloat CreateSliderInt(MVRScript self, string text, int startValue, int min, int max, bool rightSide, bool acs)
        {
            JSONStorableFloat storable = new JSONStorableFloat(text, startValue, min, max, true, true);
            storable.storeType = JSONStorableParam.StoreType.Full;
            UIDynamicSlider slider = self.CreateSlider(storable, rightSide);
            slider.label = text;
            slider.slider.wholeNumbers = true;
            slider.valueFormat = "F0";
            self.RegisterFloat(storable);
            Slider slider1 = storable.slider;
            slider1.enabled = acs;
            if (!acs) slider1.transform.parent.Find("Panel").GetComponent<Image>().color = Color.gray;
            return storable;
        }
        public static JSONStorableColor CreateColorPicker(MVRScript script, string label, Color color, bool rightSide)
        {
            HSVColor hsvColor = HSVColorPicker.RGBToHSV(color.r, color.g, color.b);
            JSONStorableColor storable = new JSONStorableColor(label, hsvColor);
            storable.storeType = JSONStorableParam.StoreType.Full;
            script.CreateColorPicker(storable, rightSide).label = label;
            script.RegisterColor(storable);
            return storable;
        }
        public static JSONStorableStringChooser CreateStringChooser(MVRScript self, string text, List<string> entries, bool rightSide, bool acs)
        {
            string defaultEntry = entries.Count > 0 ? entries[0] : "";
            JSONStorableStringChooser storable = new JSONStorableStringChooser(text, entries, defaultEntry, text);
            var popUI = self.CreateScrollablePopup(storable, rightSide);
            popUI.label = text;
            self.RegisterStringChooser(storable);
            Image image = popUI.transform.Find("Button").GetComponentInChildren<Image>();
            image.raycastTarget = acs;
            Slider slider = popUI.transform.Find("Slider").GetComponentInChildren<Slider>();
            slider.enabled = acs;
            if (!acs) image.color = Color.gray;
            return storable;
        }
        public static JSONStorableStringChooser CreateStringChooserNoRegist(MVRScript self, string text, List<string> entries, bool rightSide, bool acs)
        {
            string defaultEntry = entries.Count > 0 ? entries[0] : "";
            JSONStorableStringChooser storable = new JSONStorableStringChooser(text, entries, defaultEntry, text);
            var popUI = self.CreateScrollablePopup(storable, rightSide);
            popUI.label = text;
            Image image = popUI.transform.Find("Button").GetComponentInChildren<Image>();
            image.raycastTarget = acs;
            Slider slider = popUI.transform.Find("Slider").GetComponentInChildren<Slider>();
            slider.enabled = acs;
            if (!acs) image.color = Color.gray;
            return storable;
        }
        public static JSONStorableStringChooser CreateStringChooser(MVRScript self, string text, List<string> entries, int defaultIndex, bool rightSide, bool acs)
        {
            string defaultEntry = (defaultIndex >= 0 && defaultIndex < entries.Count) ? entries[defaultIndex] : "";
            JSONStorableStringChooser storable = new JSONStorableStringChooser(text, entries, defaultEntry, text);
            var popUI = self.CreateScrollablePopup(storable, rightSide);
            popUI.label = text;
            self.RegisterStringChooser(storable);
            Image image = popUI.transform.Find("Button").GetComponentInChildren<Image>();
            image.raycastTarget = acs;
            Slider slider = popUI.transform.Find("Slider").GetComponentInChildren<Slider>();
            slider.enabled = acs;
            if (!acs) image.color = Color.gray;
            return storable;
        }
        public static JSONStorableString CreateInfoText(MVRScript self, string text, float height, bool rightSide)
        {
            JSONStorableString storable = new JSONStorableString("Info", text);
            UIDynamicTextField textfield = self.CreateTextField(storable, rightSide);
            textfield.text = text;
            textfield.height = height;
            return storable;
        }
        public static UIDynamic CreateSpacer(MVRScript script, float height, bool rightSide)
        {
            UIDynamic spacer = script.CreateSpacer(rightSide);
            spacer.height = height;
            return spacer;
        }
        public static UIDynamicButton CreateButton(MVRScript self, string text, UnityAction callback, bool rightSide, bool acs)
        {
            UIDynamicButton btn = self.CreateButton(text, rightSide);
            btn.button.onClick.AddListener(callback);
            btn.buttonText.text = text;
            btn.button.enabled = acs;
            if (!acs)
            {
                Image image = btn.button.GetComponent<Image>();
                image.color = Color.gray;
            }
            return btn;
        }
        public static JSONStorableAction SetupAction(MVRScript self, string name, JSONStorableAction.ActionCallback callback)
        {
            JSONStorableAction action = new JSONStorableAction(name, callback);
            self.RegisterAction(action);
            return action;
        }
        public static UIDynamicLabelInput SetupTextInput(MVRScript self, string text, JSONStorableString storable, bool rightSide)
        {
            if (internalInputPrefab == null)
            {
                CreateInternalInputPrefab(self);
            }
            Transform inputTrans = internalCreateUIElement(internalInputPrefab.transform, rightSide);
            UIDynamicLabelInput dinput = inputTrans.gameObject.GetComponent<UIDynamicLabelInput>();
            storable.inputField = dinput.input;
            dinput.text.text = text;
            inputTrans.gameObject.SetActive(true);
            return dinput;
        }
        private static void CreateInternalInputPrefab(MVRScript self)
        {
            internalInputPrefab = new GameObject("LabelInput");
            internalInputPrefab.SetActive(false);
            RectTransform rt = internalInputPrefab.AddComponent<RectTransform>();
            rt.anchorMax = new Vector2(0, 1);
            rt.anchorMin = new Vector2(0, 1);
            rt.offsetMax = new Vector2(535, -500);
            rt.offsetMin = new Vector2(10, -600);
            LayoutElement le = internalInputPrefab.AddComponent<LayoutElement>();
            le.flexibleWidth = 1;
            le.minHeight = 45;
            le.minWidth = 350;
            le.preferredHeight = 45;
            le.preferredWidth = 500;
            RectTransform backgroundTransform = self.manager.configurableScrollablePopupPrefab.transform.Find("Background") as RectTransform;
            backgroundTransform = UnityEngine.Object.Instantiate(backgroundTransform, internalInputPrefab.transform);
            backgroundTransform.name = "Background";
            backgroundTransform.anchorMax = new Vector2(1, 1);
            backgroundTransform.anchorMin = new Vector2(0, 0);
            backgroundTransform.offsetMax = new Vector2(0, 0);
            backgroundTransform.offsetMin = new Vector2(0, -10);
            RectTransform labelTransform = self.manager.configurableScrollablePopupPrefab.transform.Find("Button/Text") as RectTransform;
            ;
            labelTransform = UnityEngine.Object.Instantiate(labelTransform, internalInputPrefab.transform);
            labelTransform.name = "Text";
            labelTransform.anchorMax = new Vector2(0, 1);
            labelTransform.anchorMin = new Vector2(0, 0);
            labelTransform.offsetMax = new Vector2(155, -5);
            labelTransform.offsetMin = new Vector2(5, 0);
            Text labelText = labelTransform.GetComponent<Text>();
            labelText.text = "Name";
            labelText.color = Color.white;
            RectTransform inputTransform = self.manager.configurableTextFieldPrefab.transform as RectTransform;
            inputTransform = UnityEngine.Object.Instantiate(inputTransform, internalInputPrefab.transform);
            inputTransform.anchorMax = new Vector2(1, 1);
            inputTransform.anchorMin = new Vector2(0, 0);
            inputTransform.offsetMax = new Vector2(-5, -5);
            inputTransform.offsetMin = new Vector2(160, -5);
            UIDynamicTextField textfield = inputTransform.GetComponent<UIDynamicTextField>();
            textfield.backgroundColor = Color.white;
            LayoutElement layout = textfield.GetComponent<LayoutElement>();
            layout.preferredHeight = layout.minHeight = 35;
            InputField inputfield = textfield.gameObject.AddComponent<InputField>();
            inputfield.textComponent = textfield.UItext;
            inputfield.textComponent.resizeTextForBestFit = true;
            RectTransform textTransform = textfield.UItext.rectTransform;
            textTransform.anchorMax = new Vector2(1, 1);
            textTransform.anchorMin = new Vector2(0, 0);
            textTransform.offsetMax = new Vector2(-5, -5);
            textTransform.offsetMin = new Vector2(10, -5);
            UnityEngine.Object.Destroy(textfield);
            UIDynamicLabelInput uid = internalInputPrefab.AddComponent<UIDynamicLabelInput>();
            uid.text = labelText;
            uid.input = inputfield;
        }
        public static UIDynamicTextInfo CreateInfoTextNoScroll(MVRScript self, string text, float height, bool rightSide)
        {
            if (internalTextInfoPrefab == null)
            {
                CreateInternalTextInfoPrefab(self);
            }
            Transform textTrans = internalCreateUIElement(internalTextInfoPrefab.transform, rightSide);
            UIDynamicTextInfo dtext = textTrans.gameObject.GetComponent<UIDynamicTextInfo>();
            dtext.text.text = text;
            dtext.layout.minHeight = height;
            dtext.layout.preferredHeight = height;
            textTrans.gameObject.SetActive(true);
            return dtext;
        }
        private static void CreateInternalTextInfoPrefab(MVRScript self)
        {
            internalTextInfoPrefab = new GameObject("TextInfo");
            internalTextInfoPrefab.SetActive(false);
            RectTransform rt = internalTextInfoPrefab.AddComponent<RectTransform>();
            rt.anchorMax = new Vector2(0, 1);
            rt.anchorMin = new Vector2(0, 1);
            rt.offsetMax = new Vector2(535, -500);
            rt.offsetMin = new Vector2(10, -600);
            LayoutElement le = internalTextInfoPrefab.AddComponent<LayoutElement>();
            le.flexibleWidth = 1;
            le.minHeight = 35;
            le.minWidth = 350;
            le.preferredHeight = 35;
            le.preferredWidth = 500;
            RectTransform backgroundTransform = self.manager.configurableScrollablePopupPrefab.transform.Find("Background") as RectTransform;
            backgroundTransform = UnityEngine.Object.Instantiate(backgroundTransform, internalTextInfoPrefab.transform);
            backgroundTransform.name = "Background";
            backgroundTransform.anchorMax = new Vector2(1, 1);
            backgroundTransform.anchorMin = new Vector2(0, 0);
            backgroundTransform.offsetMax = new Vector2(0, 0);
            backgroundTransform.offsetMin = new Vector2(0, -10);
            RectTransform labelTransform = self.manager.configurableScrollablePopupPrefab.transform.Find("Button/Text") as RectTransform;
            ;
            labelTransform = UnityEngine.Object.Instantiate(labelTransform, internalTextInfoPrefab.transform);
            labelTransform.name = "Text";
            labelTransform.anchorMax = new Vector2(1, 1);
            labelTransform.anchorMin = new Vector2(0, 0);
            labelTransform.offsetMax = new Vector2(-5, 0);
            labelTransform.offsetMin = new Vector2(5, 0);
            Text labelText = labelTransform.GetComponent<Text>();
            labelText.alignment = TextAnchor.UpperLeft;
            UIDynamicTextInfo uid = internalTextInfoPrefab.AddComponent<UIDynamicTextInfo>();
            uid.text = labelText;
            uid.layout = le;
            uid.background = backgroundTransform;
        }
        public static UIDynamicTextInfo CreateInfoTextNoScroll(MVRScript self, JSONStorableString storable, float height, bool rightSide)
        {
            UIDynamicTextInfo uid = CreateInfoTextNoScroll(self, storable.val, height, rightSide);
            storable.setCallbackFunction = (string text) =>
            {
                if (uid != null && uid.text != null)
                {
                    uid.text.text = text;
                }
            };
            return uid;
        }
        public delegate Transform CreateUIElement(Transform prefab, bool rightSide);
        public static void InitUI(CreateUIElement createUIElementCallback)
        {
            internalCreateUIElement = createUIElementCallback;
        }
        public static void DestroyUI()
        {
            SafeDestroy(ref internalInputPrefab);
            SafeDestroy(ref internalTextInfoPrefab);
        }
        private static void SafeDestroy(ref GameObject go)
        {
            if (go != null) GameObject.Destroy(go);
        }
        private static CreateUIElement internalCreateUIElement;
        private static GameObject internalInputPrefab;
        private static GameObject internalTextInfoPrefab;
    }
    public class UIDynamicBase : UIDynamic { }
    public class UIDynamicLabelInput : UIDynamicBase
    {
        public Text text;
        public InputField input;
    }
    public class UIDynamicTextInfo : UIDynamicBase
    {
        public Text text;
        public LayoutElement layout;
        public RectTransform background;
    }
}